from django.utils.translation import gettext_lazy as _
from django.conf import settings
from django.urls import reverse

django_site_warnings_menu_item_of_applist = {
    "title": _("Warnings"),
    "icon": "fas fa-radiation",
    "url": reverse("admin:app_list", args=["django_site_warnings"]),
    "active_patterns": ".*\/django_site_warnings\/.*",
    "permissions": ["django_site_warnings.view_warning", "django_site_warnings.view_warningcategory"],
}

django_site_warnings_menu_item_of_warning = {
    "title": _("Warnings"),
    "icon": "fas fa-radiation",
    "model": "django_site_warnings.warning",
    "permissions": ["django_site_warnings.view_warning"],
}

django_site_warnings_menu_item_of_category = {
    "title": _("Warning Categories"),
    "icon": "fa fa-boxes",
    "model": "django_site_warnings.waringcategory",
    "permissions": ["django_site_warnings.view_waringcategory"],
}

def get_default_global_sidebar(request=None):
    return [
        {
            "title": _("Home"),
            "icon": "fa fa-home",
            "url": getattr(settings, "ADMIN_SITE_HOME_URL", "/admin/"),
        },
        {
            "title": _("System Settings"),
            "icon": "fas fa-cogs",
            "children": [
                {
                    "title": _("User Manage"),
                    "icon": "fas fa-user",
                    "model": "auth.user",
                    "permissions": ["auth.view_user"],
                },
                {
                    "title": _("Group Manage"),
                    "icon": "fas fa-users",
                    "model": "auth.group",
                    "permissions": ["auth.view_group"],
                },
                django_site_warnings_menu_item_of_warning,
                django_site_warnings_menu_item_of_category,
            ]
        }
    ]



from distutils.log import warn
from django.test import TestCase
from queue import Queue

from .models import Warning
from .models import WaringCategory
from .global_sidebar import get_default_global_sidebar

notify_queue = Queue()
def send_notify(task, payload=None, force=False):
    notify_queue.put(task.pk)

Warning.register_send_notify(send_notify)

class TestDjangoSiteWarnings(TestCase):

    def setUp(self):
        self.category = WaringCategory()
        self.category.code = "category"
        self.category.name = "category"
        self.category.save()
        self.warning = Warning.make(
            category=self.category,
            title="Account not exists...",
            data="Account: test01",
            save=True,
            )

    def test01(self):
        assert len(Warning.objects.all()) == 1
        assert self.warning.category.pk == self.category.pk

    def test02(self):
        self.warning.do_task()
        assert notify_queue.get(timeout=1) == self.warning.pk

    def test03(self):
        self.warning.acknowledge()
        assert Warning.objects.all()[0].ack

        self.warning.deny()
        assert Warning.objects.all()[0].ack is False

    def test04(self):
        menus = get_default_global_sidebar()
        assert menus
        assert isinstance(menus, list)




from django.utils.translation import gettext_lazy as _

from .models import Warning


def django_site_warnings_acknowledge(modeladmin, request, queryset):
    items = []
    for item in queryset.all():
        if not item.ack:
            item.acknowledge(save=False)
            items.append(item)
    Warning.objects.bulk_update(items, fields=["ack", "ack_time"])
    message = _("{count} warnings acknowledged.").format(count=len(items))
    modeladmin.message_user(request, message)
django_site_warnings_acknowledge.short_description = _("Acknowledge selected {model_name}").format(model_name=Warning._meta.verbose_name)

def django_site_warnings_deny(modeladmin, request, queryset):
    items = []
    for item in queryset.all():
        if item.ack:
            item.deny(save=False)
            items.append(item)
    Warning.objects.bulk_update(items, fields=["ack", "ack_time"])
    message = _("{count} warnings denied.").format(count=len(items))
    modeladmin.message_user(request, message)
django_site_warnings_deny.short_description = _("Deny selected {model_name}").format(model_name=Warning._meta.verbose_name)



# Generated by Django 2.2.19 on 2021-02-26 07:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_site_warnings', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='waringcategory',
            name='code',
            field=models.CharField(max_length=255, unique=True, verbose_name='Warning Category Code'),
        ),
        migrations.AlterField(
            model_name='waringcategory',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Warning Category Name'),
        ),
    ]



import logging

from django.conf import settings

from sendmail import sendmail

logger = logging.getLogger(__name__)

def sendmail_notify(warning, payload, force):
    from_address = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_FROM")
    to_addresses = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_TO")
    server = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_SERVER")
    port = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_PORT")
    ssl = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_SSL")
    user = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_USER")
    password = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_PASSWORD")

    subject = warning.get_notify_email_subject()
    content = warning.get_notify_email_content()

    try:
        sendmail(
            from_address,
            to_addresses,
            content,
            subject,
            attachs=None,
            is_html_content=True,
            encoding="utf-8",
            charset="utf-8",
            host=server,
            port=port,
            ssl=ssl,
            user=user,
            password=password,
            )
    except Exception as error:
        logger.exception(f"send notify email failed: {error}")



# Generated by Django 2.2.19 on 2021-02-26 07:08

from django.db import migrations, models
import django.db.models.deletion
import django_safe_fields.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='WaringCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=64, unique=True, verbose_name='Warning Category Code')),
                ('name', models.CharField(max_length=64, verbose_name='Warning Category Name')),
                ('display_order', models.IntegerField(blank=True, default=0, null=True, verbose_name='Display Order')),
            ],
            options={
                'verbose_name': 'Warning Category',
                'verbose_name_plural': 'Warning Categories',
            },
        ),
        migrations.CreateModel(
            name='Warning',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(blank=True, choices=[(0, 'Task New'), (10, 'Task Ready'), (20, 'Task Doing'), (30, 'Task Done'), (40, 'Task Failed')], default=0, help_text='Task has New, Ready, Doing, Done and Failed status.', null=True, verbose_name='Status')),
                ('active', models.NullBooleanField(help_text='The task is to be runned or is running...', verbose_name='Active')),
                ('worker', models.CharField(blank=True, max_length=128, null=True, verbose_name='Worker ID')),
                ('success', models.NullBooleanField(help_text='Task success or not.', verbose_name='Success')),
                ('result', models.TextField(blank=True, null=True, verbose_name='Result')),
                ('error_code', models.IntegerField(blank=True, null=True, verbose_name='Error Code')),
                ('error_message', models.TextField(blank=True, null=True, verbose_name='Error Message')),
                ('add_time', models.DateTimeField(auto_now_add=True, help_text='Task add time is the time when the task created.', verbose_name='Add Time')),
                ('mod_time', models.DateTimeField(auto_now=True, help_text='Task modify time is the time when the task modified.', verbose_name='Modify Time')),
                ('ready_time', models.DateTimeField(blank=True, help_text='Task ready time is the time when the task set ready. Task can only start after READY.', null=True, verbose_name='Ready Time')),
                ('start_time', models.DateTimeField(blank=True, help_text='Task start time is the time when the task start to run.', null=True, verbose_name='Start Time')),
                ('expire_time', models.DateTimeField(blank=True, help_text='Task expire time is the time when the task deactived. A task is expired, it will never be runned, even it is not runned yet.', null=True, verbose_name='Expire Time')),
                ('done_time', models.DateTimeField(blank=True, help_text='Task done time is the time when the task finished .', null=True, verbose_name='Done Time')),
                ('title', django_safe_fields.fields.SafeCharField(max_length=256, verbose_name='Warning Title')),
                ('data', django_safe_fields.fields.SafeTextField(blank=True, null=True, verbose_name='Warning Data')),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_site_warnings.WaringCategory', verbose_name='Warning Category')),
            ],
            options={
                'verbose_name': 'Warning',
                'verbose_name_plural': 'Warnings',
            },
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

class DjangoSiteWarningsConfig(AppConfig):
    name = 'django_site_warnings'
    verbose_name = _("Django Site Warnings")




from magic_import import import_from_string

from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.conf import settings

from django_simpletask2.actions import reset_selected_tasks as reset_selected_tasks_core
from django_simpletask2.actions import force_do_selected_tasks as force_do_selected_tasks_core
from django_simpletask2.actions import mark_selected_tasks_done as mark_selected_tasks_done_core

from .models import Warning
from .models import WaringCategory

from .actions import django_site_warnings_acknowledge
from .actions import django_site_warnings_deny

DjangoSiteWarningsBaseAdminName = getattr(settings, "DJANGO_SITE_WARNINGS_ADMIN_BASE", "django.contrib.admin.ModelAdmin")
DjangoSiteWarningsBaseAdmin = import_from_string(DjangoSiteWarningsBaseAdminName)
if not DjangoSiteWarningsBaseAdmin:
    DjangoSiteWarningsBaseAdmin = admin.ModelAdmin

def reset_selected_tasks(*args, **kwargs):
    return reset_selected_tasks_core(*args, **kwargs)
reset_selected_tasks.short_description = _("Reset Warning Message Sending Status")

def force_do_selected_tasks(*args, **kwargs):
    return force_do_selected_tasks_core(*args, **kwargs)
force_do_selected_tasks.short_description = _("Force To Send Warning Message")

def mark_selected_tasks_done(*args, **kwargs):
    return mark_selected_tasks_done_core(*args, **kwargs)
mark_selected_tasks_done.short_description = _("Mark Select Warning Message Already Sent")

class WaringCategoryAdmin(DjangoSiteWarningsBaseAdmin):
    list_display = ["name", "code"]
    ordering = ["display_order"]

class WarningAdmin(DjangoSiteWarningsBaseAdmin):
    list_display = ["title", "add_time", "warning_status", "ack"]
    list_filter = ["category", "status", "ack"]
    readonly_fields = [] + Warning.SIMPLE_TASK_FIELDS

    def warning_status(self, obj):
        status_display_mapping = {
            Warning.READY: _("Message Not sent yet"),
            Warning.DOING: _("Message is sending"),
            Warning.DONE: _("Messgage Sent"),
        }
        return status_display_mapping.get(obj.status, _("Unknown Status"))
    warning_status.short_description = _("Warning Status")

    actions = [
        django_site_warnings_acknowledge,
        django_site_warnings_deny,
        reset_selected_tasks,
        force_do_selected_tasks,
        mark_selected_tasks_done,
    ]

admin.site.register(WaringCategory, WaringCategoryAdmin)
admin.site.register(Warning, WarningAdmin)



# Generated by Django 2.2.27 on 2022-03-10 07:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_site_warnings', '0004_auto_20210304_2119'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='warning',
            name='active',
        ),
        migrations.RemoveField(
            model_name='warning',
            name='error_code',
        ),
        migrations.RemoveField(
            model_name='warning',
            name='error_message',
        ),
        migrations.RemoveField(
            model_name='warning',
            name='expire_time',
        ),
        migrations.RemoveField(
            model_name='warning',
            name='ready_time',
        ),
        migrations.RemoveField(
            model_name='warning',
            name='result',
        ),
        migrations.RemoveField(
            model_name='warning',
            name='worker',
        ),
        migrations.AddField(
            model_name='warning',
            name='auto_reset_time',
            field=models.DateTimeField(blank=True, editable=False, help_text='After which time the task should be reset.', null=True, verbose_name='Auto Reset Time'),
        ),
        migrations.AddField(
            model_name='warning',
            name='current_step',
            field=models.IntegerField(blank=True, editable=False, null=True, verbose_name='Current Step'),
        ),
        migrations.AddField(
            model_name='warning',
            name='step1_time',
            field=models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Step1 start time'),
        ),
        migrations.AddField(
            model_name='warning',
            name='step2_time',
            field=models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Step2 start time'),
        ),
        migrations.AddField(
            model_name='warning',
            name='step3_time',
            field=models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Step3 start time'),
        ),
        migrations.AlterField(
            model_name='warning',
            name='add_time',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Add Time'),
        ),
        migrations.AlterField(
            model_name='warning',
            name='done_time',
            field=models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Done Time'),
        ),
        migrations.AlterField(
            model_name='warning',
            name='mod_time',
            field=models.DateTimeField(auto_now=True, verbose_name='Modify Time'),
        ),
        migrations.AlterField(
            model_name='warning',
            name='start_time',
            field=models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Start Time'),
        ),
        migrations.AlterField(
            model_name='warning',
            name='status',
            field=models.IntegerField(choices=[(10, 'Ready'), (20, 'Doing'), (30, 'Done')], default=10, editable=False, verbose_name='Status'),
        ),
        migrations.AlterField(
            model_name='warning',
            name='success',
            field=models.NullBooleanField(editable=False, verbose_name='Success Status'),
        ),
    ]



import os

bind = ["0.0.0.0:8061"]
workers = 2
threads = 40
daemon = True
accesslog = os.path.abspath(os.path.join(os.getcwd(), "../logs/gunicorn.access.log"))
errorlog = os.path.abspath(os.path.join(os.getcwd(), "../logs/gunicorn.error.log"))
keepalive = 300
timeout = 300
graceful_timeout = 300
loglevel = "info"



"""
Django settings for django_site_warnings_server project.

Generated by 'django-admin startproject' using Django 2.2.19.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

DJANGO_SITE_WARNINGS_REDIS_PASSWORD = os.environ.get("DJANGO_SITE_WARNINGS_REDIS_PASSWORD", "")
DJANGO_SITE_WARNINGS_REDIS_HOST = os.environ.get("DJANGO_SITE_WARNINGS_REDIS_HOST", "127.0.0.1")
DJANGO_SITE_WARNINGS_REDIS_PORT = os.environ.get("DJANGO_SITE_WARNINGS_REDIS_PORT", 6379)
DJANGO_SITE_WARNINGS_REDIS_DB = os.environ.get("DJANGO_SITE_WARNINGS_REDIS_DB", 0)

DJANGO_SITE_WARNING_SITE_NAME = os.environ.get("DJANGO_SITE_WARNING_SITE_NAME", "Django Site Warning")
DJANGO_SITE_WARNING_NOTIFY_MAIL_FROM = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_FROM", "root@localhost")
DJANGO_SITE_WARNING_NOTIFY_MAIL_TO = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_TO", "root@localhost")
DJANGO_SITE_WARNING_NOTIFY_MAIL_SERVER = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_SERVER", "localhost")
DJANGO_SITE_WARNING_NOTIFY_MAIL_PORT = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_PORT", 25)
DJANGO_SITE_WARNING_NOTIFY_MAIL_SSL = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_SSL", "true").lower() in ["true", "1", "yes", "y"] and True or False
DJANGO_SITE_WARNING_NOTIFY_MAIL_USER = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_USER", "root@localhost")
DJANGO_SITE_WARNING_NOTIFY_MAIL_PASSWORD = os.environ.get("DJANGO_SITE_WARNING_NOTIFY_MAIL_PASSWORD", "")

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.getcwd()


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'DiIHYfMZVkBc9O24NDw8QyZN4bBIqJUsWgHGIaUz5dkta1rUIQYaX8pDb6zc2kMR'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'django_site_warnings',
    'django_admin_global_sidebar',
    'django_horizontal_list_filter',
    'django_server_controller',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

from django_app_requires import add_requires
INSTALLED_APPS = add_requires(INSTALLED_APPS)

DJANGO_ADMIN_GLOBAL_SIDEBAR_MENUS = "django_site_warnings.global_sidebar.get_default_global_sidebar"

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'django_site_warnings_server.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'django_site_warnings_server.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": f"redis://:{DJANGO_SITE_WARNINGS_REDIS_PASSWORD}@{DJANGO_SITE_WARNINGS_REDIS_HOST}:{DJANGO_SITE_WARNINGS_REDIS_PORT}/{DJANGO_SITE_WARNINGS_REDIS_DB}?decode_responses=True",
        
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'zh-hans'

TIME_ZONE = 'Asia/Shanghai'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

WSGI_SERVER_ENGINE = "gunicorn"

try:
    from django_site_warnings_server_local_settings import *
except ImportError:
    pass



# Generated by Django 2.2.19 on 2021-03-04 13:19

from django.db import migrations
import django_safe_fields.fields


class Migration(migrations.Migration):

    dependencies = [
        ('django_site_warnings', '0003_auto_20210226_1617'),
    ]

    operations = [
        migrations.AlterField(
            model_name='warning',
            name='title',
            field=django_safe_fields.fields.SafeCharField(max_length=1024, verbose_name='Warning Title'),
        ),
    ]



import typing
from django.conf import settings

from django.db import models
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from django.template.loader import render_to_string

from django_simpletask2.models import SimpleTask
from django_safe_fields.fields import SafeCharField
from django_safe_fields.fields import SafeTextField

from django_site_warnings.send_notify import sendmail_notify

SIMPLE_TASK_INSTANCE = SimpleTask
PAYLOAD_TYPE = typing.Dict[str, typing.Any]
FORCE_TYPE = bool
SEND_NOTIFY_CALLBACK_TYPE = typing.Callable[[SIMPLE_TASK_INSTANCE, typing.Optional[PAYLOAD_TYPE], typing.Optional[FORCE_TYPE]], None]

class WaringCategory(models.Model):
    code = models.CharField(max_length=255, unique=True, verbose_name=_("Warning Category Code"))
    name = models.CharField(max_length=255, verbose_name=_("Warning Category Name"))
    display_order = models.IntegerField(default=0, null=True, blank=True, verbose_name=_("Display Order"))

    class Meta:
        verbose_name = _("Warning Category")
        verbose_name_plural = _("Warning Categories")

    def __str__(self):
        return self.name

    @classmethod
    def get(cls, code):
        if isinstance(code, WaringCategory):
            return code
        try:
            return cls.objects.get(code=code)
        except cls.DoesNotExist:
            category = cls()
            category.code = code
            category.name = code
            category.display_order = 99999
            category.save()
            return category

class Warning(SimpleTask):

    send_notify_callbacks = []

    category = models.ForeignKey(WaringCategory, on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Warning Category"))
    title = SafeCharField(max_length=1024, verbose_name=_("Warning Title"))
    data = SafeTextField(null=True, blank=True, verbose_name=_("Warning Data"))
    ack = models.BooleanField(default=False, verbose_name=_("Acknowledged"))
    ack_time = models.DateTimeField(null=True, blank=True, verbose_name=_("Acknowledged Time"))

    class Meta:
        verbose_name = _("Warning")
        verbose_name_plural = _("Warnings")

    def __str__(self):
        return self.title

    def get_notify_email_subject(self):
        site_name = getattr(settings, "DJANGO_SITE_WARNING_SITE_NAME", "Site Warning")
        template = getattr(settings, "DJANGO_SITE_WARNING_NOTIFY_MAIL_SUBJECT", _("[{site_name}]{title}"))
        return template.format(site_name=site_name, title=self.title)

    def get_notify_email_content(self):
        site_name = getattr(settings, "DJANGO_SITE_WARNING_SITE_NAME", "Site Warning")
        return render_to_string("django_site_warnings/notify.html", {
            "title": site_name,
            "subtitle": _("Site Warning"),
            "category": self.category.name,
            "content": self.title,
            "extra_data": self.data,
            "create_time": self.add_time,
        })

    @classmethod
    def register_send_notify(cls, send_notify:SEND_NOTIFY_CALLBACK_TYPE):
        if not send_notify in cls.send_notify_callbacks:
            cls.send_notify_callbacks.append(send_notify)

    def do_task_main(self, payload=None, force=False):
        for send_notify in self.send_notify_callbacks:
            send_notify(self, payload, force)
        return True

    @classmethod
    def make(cls, category, title, data=None, save=False):
        category = WaringCategory.get(category)
        try:
            instance = cls.objects.get(ack=False, category=category, title=title)
            return None
        except cls.DoesNotExist:
            instance = cls()
            instance.category = category
            instance.title = title
            instance.data = data
            if save:
                instance.save()
            return instance

    @classmethod
    def makemany(cls, category, warnings, save=False):
        warning_instances = []
        for warning in warnings:
            if isinstance(warning, (list, tuple)):
                data = warning[1]
                warning = warning[0]
            else:
                data = None
            instance = Warning.make(category, warning, data=data, save=False)
            if instance:
                instance.ready(save=False)
                warning_instances.append(instance)
        if save:
            Warning.objects.bulk_create(warning_instances)
        return warning_instances

    def acknowledge(self, save=True):
        self.ack = True
        self.ack_time = timezone.now()
        if save:
            self.save()
    
    def deny(self, save=True):
        self.ack = False
        self.ack_time = None
        if save:
            self.save()

Warning.register_send_notify(sendmail_notify)



# Generated by Django 2.2.19 on 2021-02-26 08:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_site_warnings', '0002_auto_20210226_1541'),
    ]

    operations = [
        migrations.AddField(
            model_name='warning',
            name='ack',
            field=models.BooleanField(default=False, verbose_name='Acknowledged'),
        ),
        migrations.AddField(
            model_name='warning',
            name='ack_time',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Acknowledged Time'),
        ),
    ]
